unit Analyze;

interface

const maxdata = 3599;

type dataarraytype = array[0..maxdata] of integer;
     floatdataarraytype = array[0..maxdata] of double;

var numentries: integer;
    analyzedataname: string;
    hand,targ,curs: floatdataArrayType;
    mhand,mcurs,mtarg,merr,mperc, mdelp: double;
    modelhandle, modelcursor, modelpercep, modeldelperc,
    modelerror,fiterr: floatdataarraytype;
    GainSet: double;
    DelaySet: integer;
    RefSet: integer;
    LeakSet: double;
    pp: double;
    expanded: boolean;
    startdata: integer;
    savediff, savedist, savename: string;

procedure plotdata;
procedure initAnModel;
procedure runmodel;
procedure autofit;

implementation

uses TrkAn2, SysUtils, classes, graphics;


const maxvariables = 5;
      maxcells = 10;


var datafile: text;
    delaybuffer: array[0..255] of double;
    inptr, outptr, timelag,t, modelref: integer;
    leak, gain,dt: double;
    rmserr,oldrms: double;
    hcenter, vcenter: integer;


function RGB(r,g,b: integer): Tcolor;
begin
result := b shl 16 + g shl 8 + r;
end;

procedure showdata(var y: floatdataArrayType);
var oldx,oldy,i,indx: integer;
    scale: double;
begin
 oldx := 0;
 oldy := 0;

 with trkanform,canvas do
  begin
   if expanded then scale := 1
   else
   scale := numentries/clientwidth;
   for i := 0 to clientwidth - 1 do
    begin
     moveto(oldx, vcenter - oldy);
     oldx := i;
     indx := startdata + round(scale *i);
     if indx >= numentries then indx := numentries - 1;
     oldy := round(y[indx]);
     if i > 0 then
     lineto(oldx, vcenter - oldy);
    end;
  end;
end;

procedure loaddata;
var varname: string;
    varval: string;
    i: integer;

procedure readindata(var data: floatdataarraytype; name:string);
begin
  i := 0;
  readln(datafile,varval);
  repeat
   if varval <> name then
    begin
     delete(varval,1,9);   //skip time entry in each line
     data[i] := strtoint(varval); // convert data entry
     readln(datafile,varval);
     inc(i);
    end;
   until (varval = name) or (EOF(datafile));
   if i < numentries then numentries := i;
end;

// body of loaddata
begin
 assignfile(datafile,AnalyzeDataName);
 numentries := maxint;
 {$I-}
 reset(datafile);
 {$I+}
 if IOResult = 0 then
 begin
  repeat readln(datafile, varname);
  until varname = 'Real Handle';
  readindata (hand,'Target');
  readindata(targ,'Real Qc');
  readindata(curs,'End');
  readln(datafile,savediff);
  readln(datafile,savedist);
  readln(datafile,savename);
  close(datafile);
 end
 else
  halt;
end;

procedure initAnModel;
begin
 loaddata;
 with trkanform, canvas do
 begin
  vcenter := clientheight div 2;
  hcenter := clientwidth div 2;
  GainSet := EditGain.value;
  DelaySet := EditDelay.Value;
  LeakSet := EditLeak.Value;
  RefSet := EditRef.Value;
 end;
 dt := 0.016667;
 modelhandle[0] := 0;
 modelref := 0;
 mhand := 0.0;
 t := 0;

end;

procedure selectVariables;
begin

end;

procedure partitionVariables;
begin
end;

procedure findBestCell;
begin
end;

procedure subdivideCell;
begin
end;

procedure calcerr;
var i: integer;
begin
 rmserr := 0.0;
 for i := 0 to numentries - 1 do
  begin
   rmserr := rmserr + sqr(fiterr[i]);
  end;
 rmserr := 100.0*sqrt(rmserr/numentries)/pp;
 with trkanform,Canvas do
 begin
  font.color := clWhite;
  TextOut(535,0,floattostrf(rmserr,fffixed,5,3) + '%');
 end;
end;

procedure plotdata;
begin
 with trkanform,canvas do
 begin
  fillrect(rect(0,0,clientwidth,clientheight));
  pen.color := rgb(255,150,0);
  font.color := pen.Color;
  showdata(analyze.modelhandle);
  textout(10,0,'MODEL');
  pen.color := clAqua;
  showdata(hand);
  font.color := pen.Color;
  textout(10,15,'PERSON');
  textout(10,30,'Difficulty ' + savediff);
  textout(10,45,'Disturbance ' + savedist);
  textout(10,60,savename);
  pen.color := clWhite;
  showdata(fiterr);
  //pen.color := clRed;
  //showdata(targ);
  pen.color := clGray;
  moveto(0,vcenter);
  lineto(clientwidth - 1,vcenter);
  oldrms := rmserr;
  calcerr;
 end;
end;

function transportlag(p: double; timelag: integer): double;
begin
 inptr := (inptr + 1) and 255;
 delaybuffer[inptr] := p;
 outptr := (inptr - timelag + 256) and 255;
 result := delaybuffer[outptr];
end;

procedure runmodel;
var t: integer;
begin
 fillchar(delaybuffer,sizeof(delaybuffer),0);
 leak := 0.01*LeakSet;
 gain := 0.1*gainset;
 timelag := delayset;
 modelref := refset div 10;
 mhand := hand[0];
 mcurs := mhand;
 t := 0;
 while t < numentries - 1 do
 begin
  mperc := mcurs - targ[t];
  mdelp := transportlag(mperc,timelag);
  merr := modelref - mdelp;
  if merr > 1000 then merr := 1000
  else
  if merr < -1000 then merr := -1000;
  mhand := mhand + (gain*merr - leak*mhand)*dt;
  mcurs := mhand;
  modelpercep[t] := mperc;
  modeldelperc[t] := mdelp;
  modelcursor[t] := mcurs;
  modelhandle[t] := mhand;
  fiterr[t] := mhand - hand[t];
  inc(t);
 end;
end;

procedure autofit;
var its: integer;
    range,delta: double;
begin
 with TrkAnForm, canvas do
 begin
  range := editgain.maxvalue - editgain.minvalue;
  delta := range/3;
  editgain.value := 0;
  its := 0;
  gainset := 0;
  runmodel;
  plotdata;
  with editgain do
  repeat
   if oldrms >= rmserr then gainset := gainset + delta
   else
     delta := -delta /2.0;
   runmodel;
   plotdata;
   caption := inttostr(round(gainset));
   inc(its);
  until (abs(delta) < 0.001*range) or (its > 100);
  editgain.value := round(gainset);
 end;
end;

end.
